module Almicube
  class Key

    ALLOWED_OPTIONS = %i[prefix suffix class_name attribute_name type distinction date selector]
    KEY_PATTERN = /%{([a-z_]+)}/ # Example) %{sample_key}

    attr_reader :options, :ranking

    attr_accessor :config

    def initialize(ranking, options={})
      @ranking = ranking
      @options = select ranking.options.merge(options)
      @config ||= Config.config
    end

    def merge!(options={})
      @options = select @options.merge(options)
      self
    end

    def [](name)
      options[name.to_sym]
    end

    def []=(name, value)
      options[name.to_sym] = value if self.class::ALLOWED_OPTIONS.include? name.to_sym
    end

    def method_missing(m, *args, &block)
      if to_str.respond_to? m
        to_str.send(m, *args, &block)
      else
        super
      end
    end

    def to_str
      key = config.key_format.clone
      key.match( self.class::KEY_PATTERN ) do |m|
        value = options.fetch(m[1].to_sym, '')
        value = value.strftime(options.fetch(:date_format, '%Y%m%d')) if value.is_a? Date
        value = value.label if value.kind_of? Selector::Base
        key.gsub! m[0], value.to_s
      end while key =~ self.class::KEY_PATTERN
      key
    end

    alias :to_s :to_str
    alias :inspect :to_str

    protected

    def select(options={})
      options.select { |k| self.class::ALLOWED_OPTIONS.include? k.to_sym }
    end
  end
end
