module Almicube
  module Ranking
    class Base
      attr_reader :selector, :bundler, :aggregator, :class_name, :date
      attr_accessor :per_page

      def initialize(options={})
        self.selector = options[:selector]
        self.aggregator = options[:aggregator] || Aggregator::SumAggregator.new
        self.bundler = options[:bundler]
        @prefix = options[:prefix]
        @suffix = options[:suffix]
        @date = options[:date] || Date.today
        @class_name = options[:class_name]
        @class_name = @class_name.constantize if @class_name.is_a? String
        @per_page = options[:per_page] || 10
      end

      def records
        selector.records
      end

      def bundled_keys
        bundler.keys || []
      end

      def connection
        Almicube::Config.config.connection
      end

      def page(page=1)
        page = [1, page.to_i].max
        revrange = connection.zrevrange(key, (page - 1) * per_page, page * per_page - 1)
        records = class_name.where("id IN (?)", revrange)
        records.each { |record| record.current_ranking = self }
        revrange.inject([]) { |l, v| l << records.detect { |r| r.to_param == v } }
      end

      def selector=(value)
        @selector = value
        @selector.ranking = self unless @selector.nil?
      end

      def aggregator=(value)
        @aggregator = value
        @aggregator.ranking = self unless @aggregator.nil?
      end

      def bundler=(value)
        @bundler = value
        @bundler.ranking = self unless @bundler.nil?
      end

      def cogs
        cogs = []
        cogs << @selector unless @selector.nil?
        cogs << @aggregator unless @aggregator.nil?
        cogs << @bundler unless @bundler.nil?
        cogs
      end

      def aggregate!(options = {})
        cogs.each { |cog| cog.before_aggregate }
        @aggregator.aggregate self
        cogs.each { |cog| cog.after_aggregate }
      end

      def aggregate(options = {})
        aggregate! options
        true
      rescue Exception => e
        false
      end

      def data?
        false
      end

      def score(item)
        ( connection.zscore key, item.to_param ) || @options.fetch(:default_score, 0)
      end

      def rank(item)
        connection.zrevrank(key, item.to_param) + 1
      end

      def incr(item, score = 1)
        connection.zincrby key, score, item.to_param
      end

      protected

      attr_reader :prefix, :suffix
    end
  end
end
