require "redis"
require "active_support/all"

module Almicube
  module Ranking
    class DataRanking < Base
      class << self
        def default_options
          { attribute_name: :score,
            prefix: 'ranking',
            suffix: '',
            distinction: '%{date}',
            date_format: '%Y%m%d',
            date: Date.today,
            per_page: 10,
            as: Float,
            default_score: 0,
            class_name: nil }
        end
      end

      def initialize(options={})
        @options = self.class.default_options.merge options.symbolize_keys

        raise TypeError, ":as option is only allowed Integer or Float" unless [Integer, Float].include? @options[:as]

        super(@options)

        @selector ||= Selector::AllSelector.new self.options
        @options.delete(:selector)
        @selector.ranking = self

        @options[:aggregator] ||= Aggregator::SumAggregator.new
        @options[:aggregator].ranking = self
      end

      KEY_PATTERN = /%{([a-z_]+)}/ # Example) %{sample_key}

      def key(options={})
        Almicube::Key.new self, options.merge( type: :data )
      end

      def data_key
        Almicube::Key.new self, @options.merge( type: :data )
      end

      def attribute_name
        @options.fetch(:attribute_name, :score).to_s.to_sym
      end

      def date
        @options.fetch(:date, Date.today)
      end

      def in(target)
        association_name = class_name.to_s.underscore.pluralize

        if ( target.class != class_name ) && target.respond_to?(association_name)
          _selector = Selector::HasManySelector.new target: target, association_name: association_name
          _selector.ranking = self
          return SubRanking.new self, selector: _selector
        end

        self
      end

      def ranged(name)
        SubRanking.new(self).ranged(name)
      end

      def options
        @options.select{ |k| k != :type }.merge({ selector: selector, aggregator: aggregator })
      end

      def data?
        selector.data_provider?
      end

      def exists?
        connection.exists key
      end

      def default_score
        @options.fetch(:default_score, 0)
      end
    end
  end
end
